# Un éléphant qui trompe énormément

Postgresql, PgAdmin, PgModeler, Postgis, qui fait quoi dans la meute ?

**Postgresql** est un systéme de base de donnée relationnel, c'est à dire que c'est une application distribuable sur différents OS qui permet de gérer des bases de données. Il existe plein d'autres technologies similaire comme sqlite, Oracle, MySql, AWS Aurora, ...

**PgAdmin** est un outil qui permet de simplifier la consultation et l'administration de vos bases postgres, ainsi que de consulter des statistiques d'utilisation facilement. C'est un otil similaire à sqlDeveloper pour Oracle, ou Dbeaver

**PgModeler** est un outil de conception graphique de base de données, il permet de modéliser facilement, vos tabes, vues ... et de visualiser facilement les relations entre les entités.

**Postgis** est une extension pour le moteur Postgresql qui permet la gestion de données spatiales. C'est actuellement l'extension la plus performante et complète sur les outils disponibles. Il existe des équivalences pour d'autres SGBD, comme le spatialite pour sqlite ou l'oracle Spatial.

![pg logos](./images/pg_logos.png)

# Concepts de base des bases de données

## Qu'est-ce qu'une base de données ?

Une base de données est une collection organisée de données interdépendantes, stockées de manière structurée et accessible. Elle permet de stocker, gérer, organiser et récupérer efficacement des informations. Une base de données est composée de tables qui regroupent des enregistrements contenant des données spécifiques. Chaque table est constituée de colonnes (aussi appelées champs) qui définissent les types de données stockées, et chaque enregistrement est représenté par une ligne. Les bases de données relationnelles, telles que MySQL, Oracle et PostgreSQL, utilisent le modèle relationnel, dans lequel les données sont organisées en tables interconnectées par des relations basées sur des clés. Ce modèle offre une structure solide pour la gestion des données, garantissant l'intégrité et la cohérence.

Cependant, avec l'émergence de nouvelles exigences en termes de volume de données, de flexibilité et de performances, les bases de données NoSQL (Not Only SQL) ont gagné en popularité. Contrairement aux bases de données relationnelles, les bases de données NoSQL, telles que MongoDB, Cassandra et Redis, adoptent des modèles de données non relationnels et évolutifs. Elles offrent une grande flexibilité pour stocker et traiter des données non structurées, semi-structurées et en constante évolution, comme les données JSON ou les données provenant de capteurs IoT. Les bases de données NoSQL sont conçues pour gérer des charges de travail distribuées à grande échelle, offrant une évolutivité horizontale et une haute disponibilité.

Le choix entre une base de données relationnelle et une base de données NoSQL dépend des besoins spécifiques de votre application. Les bases de données relationnelles sont idéales pour les applications nécessitant une structure de données rigide et une intégrité transactionnelle, comme les systèmes de gestion d'entreprise. Les bases de données NoSQL conviennent davantage aux applications nécessitant une évolutivité horizontale, une flexibilité de schéma et une gestion de données non structurées, comme les applications Web en temps réel et les plateformes d'analyse de Big Data.

## Qu'est ce qu'un SGBD ? 

Un SGBD (Système de Gestion de Base de Données) est un logiciel conçu pour gérer, stocker, organiser et manipuler de manière efficace les données dans une base de données. Il fournit une interface entre les utilisateurs et les données, permettant la création, la modification, la consultation et la suppression des données de manière sécurisée et structurée.

Un SGBD offre un ensemble de fonctionnalités qui permettent de gérer les données de manière efficace. Il permet la création de schémas pour définir la structure des données, la définition de contraintes pour garantir l'intégrité des données, la manipulation des données à travers des requêtes et des opérations, et la gestion des droits d'accès pour contrôler la sécurité des données.

Les SGBD prennent en charge différents modèles de données, tels que les modèles relationnels, les modèles hiérarchiques, les modèles réseau et les modèles orientés objet. Ils utilisent des langages de requête, tels que SQL (Structured Query Language), pour interagir avec les données et effectuer des opérations de recherche, d'insertion, de mise à jour et de suppression.

Les SGBD offrent également des mécanismes de gestion de la concurrence et de récupération en cas de pannes, garantissant la fiabilité et la disponibilité des données. Ils peuvent prendre en charge des fonctionnalités avancées telles que la réplication, la sauvegarde et la restauration, la gestion des transactions, les index pour l'optimisation des performances, et bien plus encore.

## Pourquoi utiliser PostgreSQL ?

PostgreSQL est un système de gestion de base de données relationnelle open source qui offre de nombreux avantages par rapport à d'autres technologies relationnelles. L'un des points forts de PostgreSQL réside dans la diversité dans ses extensions, qui étend ses fonctionnalités pour prendre en charge les données géospatiales notamment.

PostgreSQL présente plusieurs autres avantages significatifs. Tout d'abord, sa grande conformité aux normes SQL garantit une compatibilité élevée avec les applications existantes et facilite l'interchangeabilité avec d'autres SGBDR. Cela permet aux utilisateurs de migrer facilement depuis d'autres systèmes ou de développer des applications multiplateformes sans rencontrer de problèmes majeurs de compatibilité, via l'utilisation d'une couche d'abstraction comme DBAL en PHP par exemple.

De plus, PostgreSQL se distingue par sa robustesse et sa fiabilité. Il est réputé pour sa capacité à gérer des charges de travail intensives et de grandes quantités de données, tout en maintenant des performances élevées. Sa conception modulaire permet une extensibilité facile, ce qui signifie que de nouvelles fonctionnalités peuvent être ajoutées à la base de données sans compromettre sa stabilité.

Un autre avantage clé de PostgreSQL est sa riche gamme de fonctionnalités avancées. Outre la prise en charge des données géospatiales avec PostGIS, PostgreSQL offre des fonctionnalités telles que les transactions ACID (Atomicité, Cohérence, Isolation, Durabilité), les vues matérialisées, les déclencheurs, les procédures stockées et les index avancés. Ces fonctionnalités permettent aux développeurs de créer des applications sophistiquées et d'optimiser les performances de leurs requêtes.

Enfin, PostgreSQL bénéficie d'une communauté active et engagée. La communauté PostgreSQL travaille constamment à l'amélioration du système, à la correction des bogues, à l'ajout de nouvelles fonctionnalités et à la fourniture d'un support solide. Cette communauté dynamique offre un accès à une vaste base de connaissances, des forums d'entraide et une documentation complète, ce qui facilite l'apprentissage et le développement avec PostgreSQL.

Dans l'ensemble, PostgreSQL se distingue par sa polyvalence, sa fiabilité, ses fonctionnalités avancées et son support géospatial avec PostGIS. Ces avantages font de PostgreSQL un choix solide pour les applications nécessitant une gestion de données relationnelle robuste et une prise en charge des données géographiques.

## Utilisation des SGBD par Veremes

Toutes nos applications reposent sur un socle postgres, associé au extension Postgis et PgCrypto à l'instant cette documentation est écrite.

Vitis peut exploiter des bases Oracle pour renseigner des sources de données pour les listes déroulantes.

Dans le cas de GTF, une base sqlite est également exploitée dans certains cas.

FME permet de manipuler la quasi totalité des différent SGBDR ou noSQL.

## Terminologies courantes

- Table : Une table est une structure fondamentale dans une base de données relationnelle. Elle est composée de lignes et de colonnes et représente une entité ou un concept spécifique. Par exemple, une table "Utilisateurs" peut contenir des informations telles que les noms, les adresses e-mail et les identifiants des utilisateurs.

- Colonnes : Les colonnes, également appelées attributs ou champs, représentent les différentes caractéristiques ou propriétés d'une table. Chaque colonne a un nom unique et un type de données spécifique qui définit le type de valeurs qu'elle peut contenir. Par exemple, une table "Produits" peut avoir des colonnes telles que "Nom", "Prix" et "Quantité".

- Schéma : Un schéma est une collection logique d'objets de base de données, tels que des tables, des vues, des procédures stockées, etc. Il permet d'organiser et de regrouper les objets connexes au sein d'une base de données. Les schémas aident à maintenir la séparation et la cohérence des données dans un environnement de base de données complexe.

- Trigger (Déclencheur) : Un trigger est un objet de base de données qui est associé à une table et est déclenché automatiquement lorsque certaines actions sont effectuées sur cette table, telles que l'insertion, la mise à jour ou la suppression de données. Les triggers permettent d'effectuer des actions spécifiques avant ou après l'exécution des opérations sur la table, offrant ainsi une flexibilité pour l'automatisation des tâches et l'application de règles métier.

- Séquence : Une séquence est un objet de base de données qui génère automatiquement une série de valeurs numériques dans un ordre croissant ou décroissant. Les séquences sont souvent utilisées pour générer des identifiants uniques pour les tables, garantissant ainsi l'unicité des valeurs dans une colonne spécifique. Elles sont couramment utilisées dans les bases de données relationnelles pour gérer les valeurs d'auto-incrémentation.

- Vue : Une vue est une représentation virtuelle d'une ou plusieurs tables dans une base de données. Elle permet de sélectionner et de présenter un sous-ensemble des données d'origine selon des critères spécifiques, sans modifier les données réelles. Les vues offrent une couche d'abstraction et facilitent la simplification des requêtes complexes en fournissant des vues pré-définies et filtrées des données.

- Vue matérialisée : Une vue matérialisée est une vue qui est physiquement stockée en tant que table dans la base de données. Contrairement aux vues classiques, qui sont générées à la volée lors de l'exécution des requêtes, les vues matérialisées sont précalculées et mises à jour périodiquement pour refléter les modifications des données sous-jacentes. Cela permet d'améliorer les performances des requêtes répétitives et de réduire la charge sur la base de données.

- Rôle de connexion : Un rôle de connexion, également appelé utilisateur, est un compte qui permet à un utilisateur d'accéder à une base de données et d'effectuer des opérations sur les objets de cette base de données. Les rôles de connexion ont des privilèges et des autorisations qui définissent ce que l'utilisateur peut faire dans la base de données, tels que la lecture, l'écriture, la création ou la suppression de données.

- Rôlegroup : Un rôlegroup est un groupe de rôles de connexion qui permet de regrouper plusieurs rôles en une seule entité logique. Les rôlegroups facilitent la gestion des autorisations en attribuant des privilèges communs à un groupe de rôles, plutôt que de les définir individuellement pour chaque rôle de connexion. Cela simplifie la gestion des autorisations et facilite les modifications globales des privilèges pour un ensemble de rôles.

- Fonction : Une fonction, également appelée procédure stockée, est un bloc de code réutilisable qui effectue une tâche spécifique dans une base de données. Les fonctions sont utilisées pour encapsuler des séquences d'instructions SQL et de logique de programmation dans un seul objet. Elles peuvent être appelées à partir de requêtes SQL ou d'autres procédures stockées pour effectuer des calculs, des manipulations de données complexes ou des opérations personnalisées.

![arbre postgres](./images/postgresql_objects_tutorialdba.png)

- Lignes : Les lignes, également appelées enregistrements ou tuples, représentent des instances spécifiques de données dans une table. Chaque ligne contient les valeurs correspondantes pour chaque colonne de la table. Par exemple, une ligne dans la table "Utilisateurs" peut contenir les informations d'un utilisateur spécifique, telles que son nom, son adresse e-mail et son identifiant.

![voc 1](./images/vocabulaire_bdd1.png)

- Clé primaire : Une clé primaire est une colonne ou un ensemble de colonnes qui identifie de manière unique chaque enregistrement dans une table. Elle garantit l'unicité des données dans la table et permet de référencer facilement un enregistrement spécifique. Par exemple, une clé primaire dans la table "Utilisateurs" peut être l'identifiant unique attribué à chaque utilisateur.

- Clé étrangère : Une clé étrangère est une colonne ou un ensemble de colonnes dans une table qui fait référence à la clé primaire d'une autre table. Elle établit une relation entre les enregistrements de deux tables différentes. Par exemple, une table "Commandes" peut avoir une clé étrangère qui fait référence à la clé primaire de la table "Utilisateurs", liant ainsi les commandes à leurs utilisateurs associés.

- Index : Un index est une structure de données utilisée pour accélérer la recherche et l'accès aux données dans une table. Il est créé sur une ou plusieurs colonnes spécifiques et permet de localiser rapidement les enregistrements correspondant à certaines valeurs. Les index améliorent les performances des requêtes en réduisant le temps de recherche.

![voc 2](./images/vocabulaire_bdd2.png)


# Conception d'une base de données

Les étapes de conception du modèle de données comprennent l'identification des entités et de leurs attributs, la définition des relations, la normalisation, l'ajout de contraintes d'intégrité et la création du schéma physique. Une conception soigneuse du modèle de données est essentielle pour garantir une base de données bien structurée, cohérente et performante.

## Identification des entités et des relations

Le but de cette étape est d'identifié les différents objets nécessaire pour l'application, les différents attributs de chaque entités. Dans un second temps il faut identifier les relations qui lient chaques entités.

Le plus souvent cette étape se fait via l'analyse de documents produit par le client ou en discutant avec ce-dernier.

Une fois tous ces élément identifier, le plus simple reste de faire une première modélisation sommaire de ces objets via un diagramme UML par exemple, pour représenter les premières relations. Ce qui va permettre de commencer à discuter avec le reste de l'équipe et le client afin de confronter les différent point de vue et améliorer le modèle afin de répondre au plus proche au problème du client.
Il est important de réprésenter les cardinalités sur ce diagramme ce qui simplifie les échanges et va simplifier les prochaines étape de conception.

## Normalisation des données

A ce stade on peut commencer à abstraire le modèle pour améliorer les performances, ou le volume occupé par la base finale. Le but principal est d'éliminer les redondances tout en conservant une cohérence sur le modéle de donnée.

Les structures de base correspondantes aux cardinalités vont pouvoir se mettre en place sur le modèle.

A ce stade un outil de modélisation comme pgModeler, Visual Paradigm, ... est préférable car le modèle va s'approcher de plus en plus de sa version finale

Il y a quelques principe de base au niveau de la normalisation, ces principes ne sont pas des règles universelles. Il est possible de les adapter ou même de les ignorer dans certains cas.

- Première forme normale (1NF) : La première forme normale exige que les valeurs d'un attribut soient atomiques, c'est-à-dire qu'elles ne puissent pas être divisées en parties plus petites. De plus, chaque attribut doit être unique, sans répétitions, et chaque entité doit avoir une clé primaire unique pour l'identifier de manière univoque. Le but de cette phase et de s'assurer de la non redondances des attributs et de s'assurer que chaque entité est clairement identifiable par une clé primaire unique.

- Deuxième forme normale (2NF) : La deuxième forme normale s'applique lorsque le modèle est en 1NF et exige que chaque attribut non clé dépende complètement de la clé primaire. Cela signifie que chaque attribut doit être directement lié à la clé primaire et ne pas dépendre d'autres attributs non clés.

- Troisième forme normale (3NF) : La troisième forme normale s'applique lorsque le modèle est en 2NF et exige que chaque attribut non clé ne dépende que de la clé primaire et pas des autres attributs non clés. Cela permet d'éliminer les dépendances transitives entre les attributs non clés.

- Forme normale de Boyce-Codd (BCNF) : La forme normale de Boyce-Codd est une extension de la 3NF qui vise à éliminer les dépendances fonctionnelles non triviales. Elle exige que chaque dépendance fonctionnelle entre les attributs non clés et la clé primaire soit une dépendance complète, c'est-à-dire que la clé primaire soit nécessaire pour déterminer tous les attributs non clés. A ce stade les cardinalité sont clairement représentées par des tables ou des attributs dans le entités.

- Quatrième forme normale (4NF) : La quatrième forme normale s'applique lorsque le modèle est en BCNF et vise à éliminer les dépendances multivaluées. Elle exige que les dépendances multivaluées soient séparées en entités distinctes. On normalise les valeurs d'attribut redondantes dans des tables de référence de façon à améliorer la maintenabilité de la base de donnée.

![noramlisation cardinalité](./images/cardinalite_vitis.png)

## Structuration du modèle

Une fois le modèle de données normalisé, la quatrième étape consiste à ajouter des contraintes d'intégrité pour garantir la cohérence et la validité des données. Les contraintes peuvent inclure des règles telles que les valeurs uniques, les valeurs obligatoires, les contraintes de plage et les contraintes de référence. Par exemple, une contrainte de clé primaire garantit que chaque enregistrement dans une table est unique et identifiable, tandis qu'une contrainte de clé étrangère assure l'intégrité des relations entre les tables.

## Mise en place du modèle

Cette dernière étape consiste à déployer une première version de la base afin de pouvoir commencer à développer dessus.

La phase finale permettra de valider le fonctionnement de la base, de mettre en place les vues, régles, indexs, triggers et autres structures nécessaires pour stocker et manipuler les données de manière efficace. La mise en place des droits postgres se fera aussi à ce moment là en respectant le principe de moindre privilège.

Dans notre cas, on génère un script SQL qui sera versionné dans un fichier XML découpé par version.

## Spécificité de Vitis par rapport à la modélisation des données

Utilisation de colonnes de type timestamp with timezone pour stocker les dates. Cette contrainte est lié à la fonctionnalité de gestion des fuseaux horraire coté client. En plus Le fait d'avoir un maximum de précision sur ce genre de champ n'augmente pas la compléxité ni le volume utilisé pour stocker la donnée.

Utiliser le type jsonb à la place du type json classique, le jsonb facilite certaines fonctions des requêtes SQL classiques.

Tous les objets défini dans le modèle devront appartenir à l'utilisateur u_vitis (spécificité lié au fonctionnement des RDS sur AWS).

On ne supprime pas de colonne sur une table ou une vue (ou les tables ou vue elle-même d'ailleurs) qui a été publiée en Prod chez un client de façon à préserver la rétro-compatibilité des données.

# Manipulation des données

## Langage de requête structuré (SQL)

### Syntaxe de base

SQL est un langage standardisé utilisé pour communiquer avec les bases de données relationnelles. Il comprend différents types de commandes qui permettent de manipuler les données et de gérer la structure de la base de données. La syntaxe de base SQL se compose principalement des commandes suivantes :

- La commande SELECT est utilisée pour récupérer des données à partir d'une ou plusieurs tables. Elle permet de spécifier les colonnes à sélectionner, les tables à interroger et les critères de recherche pour filtrer les résultats.

```sql
SELECT * FROM schema.table;
SELECT chp1, chp2 FROM schema.table;
SELECT * FROM schema.table WHERE ma_condition = TRUE;
```

- La commande INSERT est utilisée pour insérer de nouvelles lignes de données dans une table spécifiée. Elle permet de spécifier les valeurs à insérer pour chaque colonne ou de récupérer les valeurs à partir d'une autre table ou d'une requête SELECT.

```sql
INSERT INTO schema.table (chp1, chp2) VALUES (chp1 chp2);
INSERT INTO schema.table (chp1, chp2) VALUES (chp1 chp2) RETURNING *;
INSERT INTO schema.table (chp1, chp2) SELECT chp1, chp2 FROM schema.table2 WHERE ma_condition;
```

- La commande UPDATE est utilisée pour modifier les données existantes dans une table. Elle permet de spécifier les colonnes à mettre à jour, les nouvelles valeurs à assigner et les critères de recherche pour identifier les lignes à modifier.

```sql
UPDATE schema.tabe SET chp1='val1' WHERE ma_condition = TRUE;
UPDATE schema.tabe SET chp1='val1' WHERE ma_condition = TRUE RETURNING *;
```

- Les commandes DELETE et TRUNCATE sont utilisées pour supprimer des lignes de données d'une table spécifiée. Elle permet de spécifier les critères de recherche pour identifier les lignes à supprimer. Contrairement à la commande DELETE, qui supprime les lignes une par une et peut être ralentie pour de grandes quantités de données, la commande TRUNCATE permet de vider rapidement une table en supprimant toutes les données d'un coup. Elle réinitialise également les valeurs des index et des séquences associées à la table.

```sql
DELETE FROM schema.table WHERE ma_condition;
TRUNCATE TABLE schema.table WHERE ma_condition;
```

- La commande CREATE est utilisée pour créer de nouvelles tables, vues, index ou autres structures dans la base de données. Elle permet de spécifier les noms, les types de données et les contraintes pour chaque colonne, ainsi que d'autres options de configuration.

- La commande ALTER est utilisée pour modifier la structure d'une table existante. Elle permet d'ajouter, de supprimer ou de modifier des colonnes, des contraintes, des index ou d'autres éléments de la table.

- La commande DROP est utilisée pour supprimer des tables, des vues, des index ou d'autres structures de la base de données.

En plus de ces commandes de base, SQL offre également des fonctionnalités avancées, telles que les clauses JOIN pour combiner des données de plusieurs tables, les fonctions d'agrégation pour effectuer des calculs sur les données, les clauses GROUP BY pour regrouper les résultats, et bien d'autres.

Chaque SGBD embarque sont interpréteur de requête SQL plus ou moins fidèle au standard.


### Clause Where et jointure

Les conditions utilisables dans les clause where et dans dans les jointures doivent retourner un booléen pour fonctionner.

La clause WHERE est couramment utilisée dans les requêtes SQL pour filtrer les résultats en fonction de certaines conditions spécifiées. Elle permet de restreindre les données renvoyées par une requête en utilisant des conditions logiques et des opérateurs de comparaison. Les conditions peuvent être basées sur les valeurs des colonnes, sur des expressions arithmétiques, sur des chaînes de caractères, ou sur d'autres critères.

Les opérateurs de comparaison couramment utilisés dans les conditions WHERE sont les suivants :

- L'opérateur d'égalité (=) permet de comparer si une valeur est égale à une autre.
- Les opérateurs de comparaison (<, >, <=, >=) permettent de comparer si une valeur est inférieure, supérieure, inférieure ou égale, ou supérieure ou égale à une autre.
- Les opérateurs d'inégalité (<>, !=) permettent de comparer si une valeur est différente d'une autre.
- L'opérateur IN permet de vérifier si une valeur fait partie d'un ensemble de valeurs spécifié.
- L'opérateur LIKE permet d'effectuer une correspondance partielle entre une valeur et un motif (pattern) spécifié à l'aide de caractères génériques (wildcards) tels que '%' et '_'.
- L'opérateur BETWEEN permet de vérifier si une valeur se situe entre deux autres valeurs.

En ce qui concerne les jointures, elles permettent de combiner des données provenant de plusieurs tables dans une seule requête. Les jointures sont généralement basées sur des relations entre les colonnes des tables, qui sont définies à l'aide de clés primaires et de clés étrangères. Les jointures les plus couramment utilisées sont les suivantes :

- La jointure INNER JOIN (ou simplement JOIN) permet de récupérer les enregistrements qui ont des correspondances dans les deux tables liées.
- La jointure LEFT JOIN (ou LEFT OUTER JOIN) permet de récupérer tous les enregistrements de la table de gauche (table de gauche dans la clause JOIN) et les enregistrements correspondants de la table de droite.
- La jointure RIGHT JOIN (ou RIGHT OUTER JOIN) permet de récupérer tous les enregistrements de la table de droite et les enregistrements correspondants de la table de gauche.
- La jointure FULL JOIN (ou FULL OUTER JOIN) permet de récupérer tous les enregistrements des deux tables, en incluant les enregistrements correspondants et non correspondants.

Les jointures sont spécifiées dans la clause FROM d'une requête SQL, en utilisant les conditions de jointure appropriées pour relier les tables entre elles. Les conditions de jointure sont généralement basées sur des clés primaires et des clés étrangères, et peuvent être combinées avec des conditions WHERE pour affiner les résultats.

![join type](./images/join_type.png)

En utilisant les conditions dans la clause WHERE et les jointures, les requêtes SQL peuvent être formulées de manière précise pour récupérer les données souhaitées à partir d'une ou plusieurs tables, en tenant compte des critères de filtrage et des relations entre les données. Cela permet d'obtenir des résultats plus pertinents et cohérents lors de l'interrogation de la base de données.

## Requêtes avancées

### Agrégation

L'agrégation de données est une fonctionnalité essentielle des requêtes SQL qui permet de regrouper et de calculer des valeurs agrégées à partir des données d'une table ou d'une requête. Elle permet d'obtenir des informations résumées et statistiques sur un ensemble de données, plutôt que sur des enregistrements individuels. Les fonctions d'agrégation les plus couramment utilisées incluent COUNT, SUM, AVG, MAX et MIN.

Lorsque vous utilisez des fonctions d'agrégation, vous pouvez également combiner des clauses GROUP BY pour regrouper les résultats en fonction de certaines colonnes. Cela permet d'effectuer des calculs agrégés pour chaque groupe distinct. Par exemple, vous pouvez regrouper les ventes par catégorie de produits et calculer le montant total des ventes pour chaque catégorie.

### Sous-requêtes

Les sous-requêtes, également appelées requêtes imbriquées, sont une fonctionnalité puissante des requêtes SQL qui permettent d'utiliser une requête à l'intérieur d'une autre requête. Elles sont principalement utilisées pour effectuer des opérations complexes et des filtres avancés en se basant sur les résultats d'une requête interne.

Une sous-requête peut être utilisée dans différentes parties d'une requête SQL, telles que la clause SELECT, la clause FROM, la clause WHERE ou la clause HAVING. Elle agit comme une requête distincte qui est exécutée en premier, puis les résultats de cette requête sont utilisés comme source de données ou de filtrage pour la requête externe.

Les sous-requêtes peuvent être utilisées de différentes manières :

Filtrage : Une sous-requête peut être utilisée dans la clause WHERE pour filtrer les résultats en fonction d'une condition spécifique. Par exemple, vous pouvez utiliser une sous-requête pour récupérer tous les clients qui ont effectué des achats supérieurs à une certaine valeur.

Comparaison : Une sous-requête peut être utilisée pour comparer une valeur avec les résultats d'une autre requête. Par exemple, vous pouvez utiliser une sous-requête dans la clause WHERE pour vérifier si un employé a un salaire supérieur à la moyenne des salaires de l'entreprise.

Valeur d'une colonne : Une sous-requête peut être utilisée pour récupérer une valeur spécifique à partir d'une autre table ou d'une autre requête. Par exemple, vous pouvez utiliser une sous-requête dans la clause SELECT pour obtenir le nombre total de commandes passées par chaque client.

Il est important de noter que les sous-requêtes peuvent impacter les performances des requêtes, surtout si elles sont mal optimisées ou si elles retournent un grand nombre de résultats. Il est recommandé de bien comprendre les concepts de sous-requêtes et de les utiliser avec parcimonie et efficacité.

Les sous-requêtes offrent une flexibilité et une puissance supplémentaires lors de l'écriture de requêtes SQL complexes. Elles permettent d'effectuer des opérations avancées en utilisant les résultats d'une requête interne, facilitant ainsi la manipulation et l'analyse de données dans une base de données.

```sql
SELECT Name
FROM Customers
WHERE CustomerID IN (
    SELECT CustomerID
    FROM Orders
)
```

### Opérations sur ensembles

Les opérations sur les ensembles sont une fonctionnalité importante des requêtes SQL qui permettent de manipuler des ensembles de résultats en combinant, comparant ou en effectuant des opérations logiques sur eux. Ces opérations sont généralement effectuées sur des ensembles de résultats obtenus à partir de requêtes distinctes.

Les opérations sur les ensembles les plus couramment utilisées incluent l'union, l'intersection et la différence.

L'opération UNION permet de combiner les résultats de deux requêtes distinctes en un seul ensemble de résultats. Elle élimine automatiquement les doublons et renvoie tous les enregistrements uniques.

L'opération INTERSECT permet de trouver les enregistrements communs à deux ensembles de résultats distincts. Elle renvoie uniquement les enregistrements qui se trouvent à la fois dans le premier et le second ensemble.

L'opération EXCEPT (ou MINUS dans certaines bases de données) permet de trouver les enregistrements qui se trouvent dans le premier ensemble mais qui ne sont pas présents dans le second ensemble. Elle renvoie uniquement les enregistrements du premier ensemble qui ne se trouvent pas dans le second ensemble.

Ces opérations sur les ensembles peuvent être utilisées pour effectuer des tâches telles que la combinaison de résultats de requêtes, la recherche d'enregistrements communs ou la suppression de résultats indésirables.

Il est important de noter que pour pouvoir effectuer ces opérations, les ensembles de résultats doivent avoir des structures de colonnes et des types de données compatibles. Les requêtes utilisées pour les opérations sur les ensembles doivent donc renvoyer des résultats avec des colonnes correspondantes.

Les opérations sur les ensembles permettent d'effectuer des manipulations complexes sur les résultats des requêtes SQL, offrant ainsi une grande flexibilité dans la récupération et le traitement des données. Elles permettent de combiner, d'interagir et de comparer facilement différents ensembles de résultats, facilitant ainsi l'analyse et la manipulation de données dans une base de données.

### Requêtes récursives

Les requêtes récursives sont une fonctionnalité avancée des requêtes SQL qui permettent de réaliser des opérations itératives en se basant sur les résultats précédents d'une requête. Elles sont utilisées pour traiter des structures de données hiérarchiques ou pour effectuer des calculs récursifs.

Une requête récursive se compose de deux parties distinctes : la partie de base et la partie récursive. La partie de base définit les résultats initiaux de la requête, tandis que la partie récursive utilise les résultats précédents pour calculer les résultats suivants.

Le fonctionnement d'une requête récursive se fait en plusieurs étapes :

- Définition de la partie de base : La partie de base de la requête est la première itération de la requête récursive. Elle est exécutée de manière similaire à une requête normale et renvoie les résultats initiaux.

- Définition de la partie récursive : La partie récursive utilise les résultats précédents de la requête pour calculer les résultats suivants. Elle est basée sur une relation de récurrence définie par le développeur. Cette partie est exécutée répétitivement jusqu'à ce qu'une condition de terminaison soit satisfaite.

- Combinaison des résultats : À chaque itération de la partie récursive, les nouveaux résultats sont combinés avec les résultats précédents pour former l'ensemble complet des résultats. Les résultats finaux sont renvoyés une fois que la condition de terminaison est atteinte.

L'utilisation de requêtes récursives est courante pour manipuler des structures de données hiérarchiques telles que les arbres, les graphes ou les relations parent-enfant. Elles permettent de parcourir et de manipuler ces structures de manière itérative en utilisant les résultats précédents.

Les requêtes récursives nécessitent une attention particulière lors de leur mise en œuvre, car une mauvaise utilisation peut entraîner des boucles infinies ou des performances médiocres. Il est important de définir une condition de terminaison appropriée pour éviter les boucles infinies et d'optimiser les requêtes récursives pour garantir des performances efficaces.

En conclusion, les requêtes récursives offrent une fonctionnalité puissante pour effectuer des calculs itératifs ou manipuler des structures de données hiérarchiques. Elles permettent d'obtenir des résultats complexes en utilisant les résultats précédents d'une requête, offrant ainsi une flexibilité et une expressivité supplémentaires dans les requêtes SQL.

```sql
WITH RECURSIVE EmployeeHierarchy AS (
    SELECT EmployeeID, Name, ManagerID, 0 AS Level
    FROM Employees
    WHERE EmployeeID = <ID_du_manager>

    UNION ALL

    SELECT E.EmployeeID, E.Name, E.ManagerID, EH.Level + 1
    FROM Employees AS E
    INNER JOIN EmployeeHierarchy AS EH ON E.ManagerID = EH.EmployeeID
)
SELECT EmployeeID, Name, Level
FROM EmployeeHierarchy
ORDER BY Level, EmployeeID;
```

### Opérateur de fenétrage

L'opérateur de fenêtrage est une fonctionnalité avancée des requêtes SQL qui permet d'effectuer des calculs et des agrégations sur des ensembles de lignes définis par une fenêtre spécifique. Il permet d'appliquer des fonctions analytiques sur un groupe spécifique de lignes sans modifier la structure globale de la requête.

L'opérateur de fenêtrage utilise la clause OVER suivie d'une clause de fenêtre pour définir les limites et les conditions de la fenêtre. Cette clause peut spécifier des critères de tri, des partitions et des clauses de délimitation pour définir la plage des lignes sur lesquelles les calculs seront effectués.

Voici un exemple pour illustrer l'utilisation de l'opérateur de fenêtrage :

Supposons que nous ayons une table "Sales" qui enregistre les ventes réalisées par différents vendeurs dans une entreprise, avec les colonnes "SalesID", "Salesperson", "Product", "Amount" et "Date". Nous voulons calculer la somme cumulative des ventes de chaque vendeur, triées par date de vente.

```sql
SELECT SalesID, vendeur, produit, montant, Date,
       SUM(montant) OVER (PARTITION BY vendeur ORDER BY Date) AS CumulativeAmount
FROM Sales
```

Dans cet exemple, nous utilisons l'opérateur de fenêtrage avec la fonction d'agrégation SUM pour calculer la somme cumulative des ventes (Amount) pour chaque vendeur (vendeur), en les triant par date de vente (Date).

La clause OVER spécifie la fenêtre de calcul, tandis que la clause PARTITION BY définit la partition des données par vendeur. Ainsi, la somme cumulative sera calculée séparément pour chaque vendeur.

En utilisant cet opérateur de fenêtrage, la requête renverra les détails des ventes (SalesID, vendeur, produit, montant, Date) ainsi que la somme cumulative (CumulativeAmount) pour chaque vendeur, avec les résultats triés par date de vente.

L'opérateur de fenêtrage est extrêmement utile pour effectuer des calculs analytiques avancés, tels que les sommes cumulatives, les moyennes mobiles, les rangs, les pourcentages, etc. Il permet de réaliser des analyses plus précises sur des ensembles de données spécifiques, sans avoir besoin de regrouper ou de modifier la structure de la requête principale.


## Indexation des données

L'indexation des données est une technique utilisée dans les bases de données pour améliorer les performances des requêtes, en particulier lors de l'exécution de requêtes SELECT. L'indexation consiste à créer des structures de données supplémentaires, appelées index, qui permettent un accès rapide et efficace aux données.

Un index est essentiellement une copie partielle des données de la table, organisée de manière à faciliter la recherche et la récupération des enregistrements. Il peut être créé sur une ou plusieurs colonnes spécifiques de la table. Lorsqu'une requête SELECT est exécutée sur une table avec un index approprié, la base de données peut utiliser cet index pour localiser rapidement les enregistrements pertinents, plutôt que de parcourir l'ensemble de la table.

L'impact de l'indexation sur une requête SELECT peut être significatif. Avec un index bien conçu, la recherche et le filtrage des données deviennent beaucoup plus rapides. Lorsqu'une condition de recherche correspond à une colonne indexée, la base de données peut utiliser l'index pour identifier rapidement les enregistrements qui satisfont la condition, réduisant ainsi le temps d'exécution de la requête.

Cependant, il est important de noter que l'indexation peut également avoir des effets négatifs sur les performances. Les index occupent de l'espace de stockage supplémentaire, ce qui peut augmenter la taille de la base de données. De plus, les index doivent être maintenus et mis à jour chaque fois que des modifications sont apportées aux données, ce qui peut entraîner un léger impact sur les opérations d'insertion, de mise à jour et de suppression.

Il est essentiel de concevoir et de gérer judicieusement les index pour obtenir un bon équilibre entre les avantages de performance et les coûts associés. Il est recommandé de créer des index sur les colonnes couramment utilisées dans les requêtes SELECT, en particulier celles qui sont souvent utilisées dans les clauses WHERE pour filtrer les enregistrements. Il est également important d'évaluer régulièrement les performances de la base de données et d'ajuster les index en conséquence pour maintenir une optimisation continue.

En conclusion, l'indexation des données joue un rôle crucial dans l'amélioration des performances des requêtes SELECT. Elle permet d'accélérer la recherche et la récupération des données en utilisant des structures d'index optimisées. Cependant, il est important de bien concevoir et gérer les index pour éviter les impacts négatifs sur les opérations de modification des données et pour maintenir des performances optimales.

PostgreSQL propose plusieurs méthodes d'indexation pour améliorer les performances des requêtes. Voici une description des principales méthodes d'indexation disponibles dans PostgreSQL :

- Index B-tree : C'est la méthode d'indexation par défaut dans PostgreSQL. Les index B-tree sont efficaces pour les opérations de recherche et de tri. Ils fonctionnent bien avec des valeurs uniques ou peu répétées. Les index B-tree sont équilibrés et permettent des recherches rapides en utilisant des opérations de comparaison, telles que l'égalité ou l'inégalité.

- Index Hash : Les index Hash utilisent une fonction de hachage pour stocker les valeurs d'index. Ils sont adaptés pour les recherches rapides sur des valeurs égalitaires, mais moins efficaces pour les opérations de tri ou de recherche basées sur des plages. Les index Hash fonctionnent bien lorsque les données sont uniformément réparties et que les valeurs d'index sont bien distribuées.

- Index GiST (Generalized Search Tree) : Les index GiST sont polyvalents et peuvent être utilisés pour différents types de recherches, tels que les recherches basées sur des plages, les opérations de texte et les opérations spatiales. Ils permettent de créer des index personnalisés en implémentant des méthodes d'indexation spécifiques pour différents types de données.

- Index GIN (Generalized Inverted Index) : Les index GIN sont utilisés pour les recherches en texte intégral et pour les opérations sur des tableaux ou des listes de valeurs. Ils sont efficaces pour les recherches de correspondance partielle, de recherche de mots-clés et d'opérations de recherche avancées.

- Index BRIN (Block Range Index) : Les index BRIN sont conçus pour les tables volumineuses contenant des données ordonnées par ordre d'insertion. Ils permettent de réduire la taille de l'index en regroupant les blocs de données contigus et en conservant les informations de début et de fin pour chaque groupe de blocs. Les index BRIN sont utiles pour les requêtes d'agrégation ou les opérations de recherche basées sur des plages.

Chaque méthode d'indexation a ses propres avantages et inconvénients, et la sélection de la méthode appropriée dépend du type de données, des opérations de requête fréquentes et des objectifs de performance spécifiques. PostgreSQL permet également de combiner différentes méthodes d'indexation pour optimiser les performances dans des scénarios plus complexes.

### Comment bien indexer une table ?

Avant de commencer à réfléchir à l'indexation d'une table, il est essentiel de comprendre comment elle va être interrogée (la structure globale des requêtes SELECT sur la table, l'utilisation qui va être faite de la donnée) et comment fonctionne le mécanisme interne du SGBD.

Sans indexation, pour une requête donnée, le SGBD devra récupérer l'ensemble des enregistrements pour vérifier la valeur correspondant à la colonne demandée dans la clause WHERE, GROUP BY ou ORDER BY. Ces clauses n'interviennent pas nécessairement dans les mêmes étapes du moteur SGBD (c'est pourquoi un LIMIT n'a plus d'effet sur les performances dès lors qu'un ORDER BY est utilisé, car ce dernier dégrade les performances puisqu'il intervient après un premier parcours de la table pour la clause WHERE).

Pour illustrer ce mécanisme, imaginons qu'une table est un livre, et chaque enregistrement une page. Il faut lire chaque page pour vérifier que l'élément recherché est présent, puis relire les pages concernées pour les trier.

Un index agit comme une table des matières pour le livre, c'est-à-dire une page supplémentaire liée au livre qui permet de retrouver plus rapidement une information spécifique.

Plus une colonne sera récurrente dans les requêtes, plus l'indexation de cette dernière sera pertinente **sous certaines conditions**.

La colonne doit être à la fois suffisamment discriminante sans trop l'être, il faut évaluer la cardinalité moyenne. Autrement dit, si une colonne comporte *n* valeurs (où *n* est bien inférieur au nombre d'enregistrements) avec une répartition égale pour chaque valeur, l'index sera très efficace. À l'inverse, si une valeur représente 95 % de la répartition dans la table, alors l'impact de l'index pourrait être discutable (cela dépendra principalement de la différence de volumétrie entre la table et l'index, ainsi que dans quelle portion se trouve votre zone de travail).

Si la colonne est régulièrement mise à jour dans la vie de l'enregistrement, l'index peut représenter un coût significatif à maintenir (par exemple, un état). D'autant plus que, dans ce cas, la colonne est souvent peu discriminante, car il n'y a que quelques états finaux. Pour reprendre l'exemple du livre, c'est comme si la table des matières devait se mettre à jour constamment au fil de la recherche, ce qui est contre-productif pour le lecteur.

Dans le cas de colonnes utilisées pour du tri, il peut être utile de prévoir un index intégrant le sens de tri préféré.

Le moteur du SGBD aura des difficultés à travailler avec plusieurs index en même temps. Imaginez un lecteur devant croiser trois tables des matières pour trouver une information, c'est quasi impossible. Il en va de même pour le moteur SGBD. En réalité, le moteur est capable d'utiliser plusieurs index, mais il fera parfois le choix de limiter cet usage, ce que l'on peut observer avec les instructions **EXPLAIN ANALYZE**. D'où l'intérêt d'utiliser des index composites (ou multi-colonnes).

En réutilisant le même exemple, il faut voir un index composite comme une table des matières de ce type : 
- Col1 - Val1
  - Col2 - Val1 ................. 0x0000...
  - Col2 - Val2 ................. 0x0000...
  - Col2 - Val3 ................. 0x0000...
- Col1 - Val2
  - Col2 - Val1 ................. 0x0000...
  - Col2 - Val2 ................. 0x0000...
  - Col2 - Val3 ................. 0x0000...

Dans le cas d'une table interrogée de manière automatique (par exemple les demandes GTF ou les workflows GTF par les moteurs), il est facile de poser une structure pour un index composite, puisque les requêtes sont toujours construites de la même manière. Cela devient plus complexe lorsque la donnée est interrogée via l'API et où l'utilisateur peut personnaliser la requête.

Le volume de la table peut également favoriser l'ajout d'index. Dès que vous avez une colonne lourde (habituellement byte array, text, jsonb), il est pertinent d'indexer les colonnes régulièrement utilisées dans les filtres. Ces colonnes étant parfois très volumineuses, sur une requête non indexée, le SGBD devra lire l'ensemble de cette donnée, même si elle n'est pas directement utilisée dans la requête (toujours avec l'exemple du livre, c'est comme lire 90 % de la page pour rien systématiquement).

Un cas concret : un livre de cuisine. Je peux chercher un type de plat (dessert, plat principal, entrée, ...), un sous-type (gâteau, plat mijoté, soupe, salade, ...), un ingrédient, un temps de préparation, etc.

Des index qui pourraient être pertinents seraient : 
- Index monocolonne sur le type, le sous-type, le temps de préparation, la difficulté.
- Index composite sur le type et le sous-type.

### Cas pratique : les workflows de GTF

Il y a plusieurs tables (wf_iteration, wf_task, workflow), la table la plus interrogée par les moteurs étant wf_task. Sur une application Dtnet de production, après un an d'exécution, la table contient 17 Go de données et inclut une colonne de type jsonb.

Au niveau de l'interface, on peut afficher le contenu des trois tables, lister les workflows, les itérations et les tasks d'une itération.

Le moteur récupère les tasks liées à un workflow_id, triées par priorité et date de création. Il cherche aussi parfois à savoir si la task est dans une chaîne de traitement synchrone (colonne chain). Il y a aussi un état sur la tâche.

1. Identification des requêtes problématiques :

PgAdmin permet de voir en temps réel les requêtes en cours d'exécution. Il faut alors analyser les temps de traitement au bon moment, ce qui peut être assez complexe. AWS RDS permet de visualiser les requêtes les plus coûteuses en CPU sur un intervalle de temps.

```sql
SELECT * FROM s_gtf.wf_task WHERE status_id=$1 AND chain IS NULL AND workflow_id IN ($2, $3) AND wf_step_id IS NOT NULL AND wf_iteration_id IS NOT NULL AND archived IS NOT true ORDER BY priority DESC NULLS LAST, creation_date ASC LIMIT ?;

SELECT * FROM s_gtf.wf_task WHERE status_id = ? AND workflow_id IN (?, ?) AND wf_step_id IS NOT NULL AND wf_iteration_id IS NOT NULL AND archived IS NOT true ORDER BY priority DESC NULLS LAST, creation_date ASC LIMIT ?
```

2. EXPLAIN ANALYZE des requêtes concernées : 

La charge sur le serveur (plusieurs processus moteur se réveillant en même temps pour interroger la table) associée au volume de données à interroger provoque l'apparition de traitements parallèles par le moteur du SGBD, ce qui provoque une surcharge du CPU sur la base.

3. Recoupement des requêtes problématiques pour identifier les colonnes indexables : 

Les colonnes revenant le plus souvent dans les requêtes: 
- workflow_id (clause WHERE)
- priority (clause ORDER BY)
- creation_date (clause ORDER BY)
- chain (clause WHERE)
- status_id (clause WHERE)
- archived (clause WHERE)

Certaines colonnes utilisées dans le WHERE étant NOT NULL mais interrogées avec l'instruction IS NOT NULL n'ont pas de sens. Il faut donc modifier la requête dans le moteur pour l'alléger.

4. Estimation de la répartition de données pour indexation : 

**workflow_id :** 

C'est une clé étrangère sur une autre table.

L'application contient plusieurs centaines de workflows (relativement à plusieurs millions de tasks), avec une répartition d'environ 75 % répartie de manière égale sur trois workflows et 25 % sur les autres workflows.

**priority :**

Colonne de type integer, les valeurs sont supérieures à 0 avec une infinité de possibilités théoriquement. Dans la pratique, il y a environ 4-5 valeurs, principalement 1 et 8, avec une répartition de 75 % pour la valeur 1 et de 25 % pour 8 (les autres valeurs représentant moins de 5 % du volume total, elles sont considérées comme négligeables).

**creation_date :**

Colonne de type timestamp.
Il y a peu de chance que la valeur soit suffisamment discriminante pour impacter la requête. En revanche, le fait de parcourir les blocs complets (soit 17 Go) pour réaliser le tri final rend l'apport de l'index non négligeable. Il pourrait donc être pertinent de l'inclure dans un index composite. D'autant plus que la colonne n'évolue pas au cours de la vie de l'enregistrement, l'index ne sera donc pas lourd à maintenir.

**chain :**

Colonne de type integer, qui permet de définir si une task est exécutée de manière synchrone avec une autre task. En gros, si c'est null, c'est que c'est une task à traiter pour une nouvelle exécution des moteurs. À l'inverse, la task est traitée par un thread existant du moteur en réponse à une autre task traitée.
Il existe donc les valeurs suivantes : NULL, 0, et tous les entiers supérieurs à 0.
Dans la pratique, lorsque cette colonne est utilisée dans le WHERE, c'est la valeur NULL qui est recherchée, représentant environ 5-15 % des tasks. Il pourrait donc être pertinent d'indexer ce champ pour ce cas bien précis, puisqu'il éliminerait la majorité des données à analyser.

Cette colonne n'étant pas dans toutes les requêtes, il faudra probablement envisager de doubler certains index, en mono-colonne et multi-colonne.

**status_id :**

L'état d'une task contient que quelques valeurs, mais la répartition est majoritairement sur un seul état. L'état changeant au cours du cycle de vie de l'enregistrement, cet index aura un coût supérieur aux autres à maintenir pour le SGBD.

**archived :**

Booléen qui définit si une task est terminée et archivée. Actuellement, plus de 90 % des tasks sont archivées. L'impact de l'index est donc discutable, cependant, si la recherche se fait du côté le plus petit de la répartition, l'index peut se justifier.

5. Création des index

Il faut évaluer l'impact d'un index et mettre en place les index les plus significatifs pour nos requêtes spécifiques.

Un premier index composite sur la clause ORDER BY semble plus pertinent : 
```sql
CREATE INDEX idx_wf_task_engine_priority_creation ON s_gtf.wf_task (priority DESC NULLS LAST, creation_date ASC); -- 526 Mo
```

Un index sur l'identifiant du workflow revenant le plus régulièrement devrait également avoir un impact significatif : 
```sql
CREATE INDEX idx_wf_task_engine_workflow_id ON s_gtf.wf_task (workflow_id); -- 116 Mo
```

Dans un second temps, il sera possible d'ajouter des index sur chain, status_id, archived, mais dans ce cas-là, il faudra vérifier le comportement de la requête avec plusieurs index mono-colonne, ce qui serait la solution la plus simple. Sinon, il faudra créer un index composite par combinaison exploitée dans les différentes requêtes des moteurs.

## Transactions

La gestion des transactions est un concept fondamental dans les systèmes de bases de données qui garantit l'intégrité et la cohérence des données. Une transaction représente une séquence d'opérations exécutées comme une unité indivisible, c'est-à-dire soit toutes les opérations sont exécutées avec succès et validées, soit aucune opération n'est exécutée du tout.

Dans une base de données, une transaction est généralement définie par une combinaison d'opérations de lecture (SELECT) et d'opérations de modification (INSERT, UPDATE, DELETE). Les transactions permettent de regrouper ces opérations et d'assurer que l'ensemble des modifications apportées aux données est cohérent et durable.

La gestion des transactions est basée sur le concept ACID, qui signifie Atomicité, Cohérence, Isolation et Durabilité :

Atomicité : Une transaction est une unité atomique qui est exécutée dans son intégralité ou pas du tout. Si une opération échoue ou rencontre une erreur, toutes les opérations précédentes sont annulées (rollback), et les données sont restaurées à leur état précédent.
Cohérence : Une transaction garantit que les données sont maintenues dans un état valide avant et après son exécution. Les contraintes et les règles d'intégrité sont respectées tout au long de la transaction.
Isolation : Les transactions s'exécutent de manière isolée les unes des autres. Cela signifie que les opérations d'une transaction en cours ne sont pas visibles pour les autres transactions jusqu'à ce qu'elles soient validées. Cela préserve l'intégrité des données et évite les problèmes de concurrence.
Durabilité : Une fois qu'une transaction est validée et confirmée, les modifications apportées aux données deviennent permanentes et survivront à d'éventuelles pannes du système.
Voici un exemple concret pour illustrer la gestion des transactions :

```sql
BEGIN TRANSACTION;
SELECT stock_quantity FROM Inventory WHERE product_id = <id_produit> INTO @stock_quantity;
UPDATE Inventory SET stock_quantity = @stock_quantity - <quantite_commandee> WHERE product_id = <id_produit>;
UPDATE Products SET stock = stock - <quantite_commandee> WHERE product_id = <id_produit>;

-- ROLLBACK; pour tout annuler
COMMIT;
```

La gestion des transactions est essentielle pour garantir la cohérence et l'intégrité des données dans les systèmes de bases de données. Elle permet d'assurer que les opérations sont exécutées avec succès ou entièrement annulées en cas d'erreur, offrant ainsi une garantie de fiabilité des données.

## Héritage de table

Supposons que nous avons une table parente appelée "Employee" contenant des attributs communs à tous les employés, tels que "id", "name" et "salary". Nous souhaitons créer deux tables filles, "Manager" et "Staff", qui héritent des attributs de la table "Employee" mais ajoutent également des attributs spécifiques à chaque type d'employé.

Voici comment cela pourrait être réalisé :

```sql
-- Création de la table parente Employee
CREATE TABLE Employee (
id SERIAL PRIMARY KEY,
name VARCHAR(100) NOT NULL,
salary DECIMAL(10, 2) NOT NULL
);

-- Création de la table fille Manager héritant de la table Employee
CREATE TABLE Manager (
department VARCHAR(50) NOT NULL,
PRIMARY KEY (id),
FOREIGN KEY (id) REFERENCES Employee (id)
) INHERITS (Employee);

-- Création de la table fille Staff héritant de la table Employee
CREATE TABLE Staff (
position VARCHAR(50) NOT NULL,
PRIMARY KEY (id),
FOREIGN KEY (id) REFERENCES Employee (id)
) INHERITS (Employee);

-- Insertion de données dans la table Manager
INSERT INTO Manager (name, salary, department) VALUES ('John Smith', 5000.00, 'Finance');

-- Insertion de données dans la table Staff
INSERT INTO Staff (name, salary, position) VALUES ('Jane Doe', 3000.00, 'Assistant');

-- Sélection des employés de toutes les tables (Employee, Manager, Staff)
SELECT * FROM Employee;

-- Sélection des employés uniquement dans la table Manager
SELECT * FROM Manager;

-- Sélection des employés uniquement dans la table Staff
SELECT * FROM Staff;
```

Dans cet exemple, nous créons d'abord la table parente "Employee" avec les attributs communs à tous les employés. Ensuite, nous créons deux tables filles, "Manager" et "Staff", en utilisant la clause INHERITS pour hériter des attributs de la table parente.

Nous insérons ensuite des données spécifiques dans les tables "Manager" et "Staff". Les clés primaires des tables filles sont également des clés étrangères faisant référence à la clé primaire de la table parente.

Enfin, nous effectuons des sélections pour récupérer les employés de toutes les tables (Employee, Manager, Staff) ainsi que les employés spécifiques à chaque table.

Le mécanisme d'héritage permet de regrouper des tables liées par une relation "est-un" et de partager les attributs communs tout en permettant des attributs spécifiques à chaque table fille. Cela facilite la gestion des données et les requêtes en permettant de traiter les tables filles comme des entités distinctes tout en profitant de l'héritage des attributs et des contraintes définis dans la table parente.

## JSONB

Le type JSONB dans PostgreSQL offre une flexibilité et une puissance significatives pour le stockage et la manipulation de données au format JSON (JavaScript Object Notation). JSONB est une extension du type JSON qui permet un stockage efficace et une manipulation avancée des données JSON.

L'utilisation du type JSONB permet de gérer des données semi-structurées et évolutives, ce qui est particulièrement utile lorsque la structure des données peut varier d'un enregistrement à un autre. Cela permet de stocker des informations diverses et complexes dans une colonne, sans avoir à définir un schéma fixe pour chaque enregistrement.

Voici quelques exemples d'utilisation du type JSONB dans PostgreSQL :

Stockage de données JSON structurées :
```sql

CREATE TABLE Employee (
  id SERIAL PRIMARY KEY,
  name VARCHAR(100),
  details JSONB
);

INSERT INTO Employee (name, details)
VALUES ('John Smith', '{"age": 35, "position": "Manager", "salary": 5000}');
```

Requêtes basées sur des valeurs JSONB :
```sql
SELECT *
FROM Employee
WHERE details->>'position' = 'Manager';
```

Mise à jour de valeurs spécifiques dans une colonne JSONB :

```sql
UPDATE Employee
SET details = jsonb_set(details, '{salary}', '5500', true)
WHERE id = 1;
-- OU

UPDATE Employee
SET details = details || '{"age" : 40}'
WHERE id = 1;

```

Utilisation de fonctions JSONB intégrées :

```sql
SELECT details->'age' AS age, jsonb_array_length(details->'projects') AS project_count
FROM Employee;
```

Le type JSONB offre des fonctionnalités avancées pour extraire, modifier et interroger des données JSON. Les opérateurs et fonctions spécifiques permettent d'accéder aux valeurs JSONB, de les manipuler, de rechercher des clés spécifiques et d'effectuer des opérations complexes sur les données.

L'utilisation du type JSONB peut être particulièrement avantageuse dans les scénarios où la structure des données est flexible ou où l'on souhaite stocker des données complexes sans avoir à définir un schéma fixe. Cela permet une adaptation facile aux changements de la structure des données et facilite la manipulation des données JSON au sein de la base de données PostgreSQL.