# Concepts de base

## Machine de Turing

Une machine de Turing est un **modèle mathématique théorique** inventé par Alan Turing en 1936. Elle est considérée comme l'un des premiers modèles de calcul, et est souvent utilisée pour illustrer les concepts de la théorie de la calculabilité.

Le fonctionnement d'une machine de Turing est relativement simple. Elle se compose d'un **ruban infini divisé en cases**, sur lequel sont **inscrits** des **symboles**. **Un mécanisme de lecture/écriture** (la tête de lecture/écriture) peut se déplacer le long du ruban, lire les symboles et en écrire de nouveaux. La machine possède également **un état interne** qui détermine sa réponse à un symbole donné.

![ruban infini](./images/ruban-turing.jpg)

Lorsque la machine est allumée, elle commence par lire le symbole se trouvant sous la tête de lecture/écriture. En fonction de l'état interne de la machine et du symbole lu, elle effectue une opération spécifique, telle que changer le symbole sur le ruban, déplacer la tête de lecture/écriture vers la gauche ou la droite, ou passer à un nouvel état interne.

La machine de Turing est un modèle de calcul universel, ce qui signifie qu'elle peut effectuer n'importe quel calcul qui peut être décrit de manière algorithmique. **En théorie, toute fonction calculable peut être exprimée en tant que machine de Turing.**

Le modèle de la machine de Turing a également conduit à la création de l'architecture de von Neumann, qui est la base des ordinateurs modernes. L'architecture de von Neumann utilise une mémoire à accès aléatoire pour stocker les programmes et les données, et une unité centrale de traitement (CPU) qui peut exécuter des instructions sur ces données.

Enfin, la machine de Turing a également des implications en théorie de la complexité. Les problèmes qui peuvent être résolus par une machine de Turing en un temps polynomial sont considérés comme faisant partie de la classe P, tandis que les problèmes qui nécessitent un temps exponentiel ou plus pour être résolus sont considérés comme faisant partie de la classe NP. La théorie de la complexité est essentielle pour comprendre les limites de la résolution de problèmes informatiques, notamment dans les domaines de la cryptographie et de la sécurité.

en résumé, Alan Turing au delà d'avoir cassé l'un des sytèmes de cryptographie les plus performant de l'histoire et permis par ses actions la victoire des alliés pendant la seconde guerre mondiale, a inventé les concepts qui régissent encore aujourd'hui toutes les machines modernes.

```{Note}
  Un film biopic 'Imitation Game' retrace l'histoire d'Alan Turing, de ses actions pendant la guerre à la fin tragique de sa vie.
  Si ça peut vous interresser de voir de manière un peu romancer l'histoire de ce génie, je ne peux que vous encourager à le voir.
```

## Fonctionnement et application

Quelques décénnies aprés la création de la machine de turing, un ordinateur, ça reste un ensemble de machine qui communiquent entre elles et occupent une place assez conséquente, pour une utilité discutable vu la compléxité de fonctionnement.

![old computers](./images/old_computers.png)

L'humanité part du principe que saisir des calculs mathématiques en binaire sans forcément être capable de lire/comprendre le résultat c'est pénible, d'où la création des applications et du métier de développeur.

Un ordinateur moderne doit être visualiser comme un empilement de couche : 

![computer layer](./images/archi_detaillee.png)


### Premier niveau : Hardware

Je ne vais pas faire une liste exhaustive des différents composants modernes constituant un PC.

En ce qui concerne le développement il y en a que trois qui nous interresse :
- le CPU : on va parler de coeurs physiques et virtuels (un coeur physique peut représenter plusieurs coeurs virtuels, on peut aussi parler de thread), grossiérement un thread, c'est une tête de lecture de la machine de Turing, donc c'est la capacité à réaliser une opération mathématique. Si j'ai 8 thread, je peux réaliser 8 opérations mathématiques en parallèle.
- la RAM : On parle aussi de mémoire vive, c'est une partie du ruban infini de la machine de turing. Le principe c'est que l'état de ce ruban n'est conservé que tant que la machine est allumée, par contre les temps d'accés et de manipulation sur cette partie du ruban est beaucoup plus rapide.
- Le disque dur : l'autre partie du ruban de la machine de turing, celle-ci quant à elle conserve son état même en cas d'arret de la machine. On va retrouver deux familles de disque les HDD (Hard Disk Drive) et les SSD (Solid State Drive), le SSD est bien plus rapide que le HDD et se rapproche de plus en plus des perfromances que l'on connait avec la RAM.

### Deuxième niveau : OS

l'OS (Operating System) c'est ce qui va rendre utilisable une machine de turing et l'intéraction avec les différents composants.

Il va agir comme un traducteur entre vos application et les composant de l'ordinateur.
Il va prendre sur lui la gestion de la mémoire, le stockage de fichiers, ...

Historiquement on part d'un noyau commun UNIX qui a ensuite dévié sur les divers OS que l'on connait : Windows (ne vient pas directement d'UNIX), Linux, Mac OS.

En ce qui nous concerne on va se cantonner au Linux moderne. Un linux c'est avant tout un noyau, c'est un ensemble de fonctionnalité communes à tous les linux, puis chaque distribution va embarquer différents paquets additionnels.

On retrouve deux grandes familles de distributions : 
- Debian
- Red Hat / Fedora

Grossièrement toutes les autres distributions vont être des dérivés avec plus ou moins de paquets additionnels. 

Exemple : 
- Ubuntu : Debian avec un certain nombre de paquet de base installé comme par exemple nano.
- Kali : Debian qui utilise KDE à la place de gnome ou unity et qui embarque tout un tas d'application de pentest.

Il faut voir une distribution linux comme un ensemble de LEGO, on va retrouver certains LEGO chez tout le monde puis chacun va embarquer un peu ce qu'il veut.

Veremes utilise essentiellement Ubuntu qui dérive de Debian (que l'on peut utiliser aussi), ce choix est induit par les préférence d'AWS qui préconise Ubuntu et est donc plus performant sur les sortie de correctifs et le suivi des nouvelles versions.

Lors des installations il faut favoriser les versions LTS (Long term support) et si possible la dernière.

### Derniers niveaux : Librairies, API et applications

Pour développer on va se retrouver à utiliser certains composants développés par des tiers.

Par exemple en C/C++ on va régulièrement utiliser des librairies dynamiques (DLL) pour s'économiser de la production de code et qui sera massivement utilisé et donc plus robuste.

On peut aussi utiliser des API, SDK, ou même des exécutables.

Par exemple si je reste dans le cas de nos applications, on va configurer un WebServer Apache, qui va lancer l'exécutable de PHP pour aller interpréter le code de API, ce qui va provoquer des actions de la machine pour produire un résultat.

## Compléments sur Linux

### système de fichiers

La racine d'une distribution linux regroupe toujours les mêmes dossiers, cahcun de ces dossier à une fonction précise:
- **bin** : contient des programmes binaires essentiels utilisés par le système d'exploitation.
- **boot** : contient les fichiers nécessaires au démarrage du système d'exploitation.
- **dev** : contient des fichiers spéciaux représentant les périphériques matériels du système, tels que les disques durs, les claviers, les souris, etc.
- **etc** : contient des fichiers de configuration système. Les fichiers dans ce répertoire sont utilisés pour configurer divers aspects du système, tels que les paramètres réseau, les comptes d'utilisateurs, certains services, etc.
- **home** : contient les dossiers personnels des utilisateurs.
- **lib** : contient des bibliothèques partagées (fichier en .so équivalent aux dll windows) nécessaires pour exécuter les programmes système.
- **media** : contient les points de montage pour les périphériques de stockage externes tels que les CD-ROM, les DVD, les clés USB, etc.
- **mnt** : contient les points de montage pour les systèmes de fichiers temporaires, tels que les disquettes, les CD-ROM, les DVD, etc.
- **opt** : contient des packages de logiciels tiers installés sur le système.
- **proc** : contient des fichiers système spéciaux qui fournissent des informations sur les processus en cours d'exécution, les périphériques matériels, la mémoire système, etc.
- **root** : contient le dossier personnel de l'utilisateur root.
- **sbin** : contient des programmes système nécessaires à l'administration du système.
- **tmp** : contient des fichiers temporaires créés par les programmes en cours d'exécution.
- **usr** : contient des programmes, des bibliothèques, des fichiers de configuration et des données pour les utilisateurs.
- **var** : contient des fichiers de données qui changent fréquemment, tels que les journaux système et les fichiers de spool pour les tâches en attente. 

La description de chaque dossier est l'usage attendu de ce dossier, pas celui qui en est fait par toute les applications.

### Variables d'environnement

- **PATH** : définit les répertoires où le système d'exploitation doit chercher les programmes exécutables. Cela permet aux utilisateurs d'exécuter des commandes en tapant simplement leur nom, sans avoir à spécifier le chemin complet vers le fichier exécutable.
- **HOME** : contient le chemin du répertoire personnel de l'utilisateur.
- **USER** : contient le nom d'utilisateur de l'utilisateur actuellement connecté.
- **PS1** : définit le format de l'invite de commande, qui est l'affichage qui apparaît lorsque l'utilisateur est invité à entrer une commande.
- **LANG** : définit la langue par défaut utilisée pour les messages d'erreur et autres messages système.
- **LD_LIBRARY_PATH** : définit le chemin vers les bibliothèques partagées utilisées par les programmes installés sur le système.
- **PWD** : contient le chemin complet du répertoire de travail actuel de l'utilisateur. Le répertoire de travail actuel est le répertoire dans lequel l'utilisateur se trouve actuellement dans le système de fichiers lorsqu'il exécute des commandes à partir de l'invite de commande.

Il existe d'autres variables, cette liste contient les variables qui sont considérées comme les plus utiles.
## Gestion de la mémoire

Lorsque l'on développe, il faut garder à l'esprit la façon dont la mémoire va être gérée. On parle habituellement du **tas** et de la **pile**.

Le tas et la pile sont deux structures de données importantes en programmation. Le **tas** est une région de la mémoire qui stocke les **objets dynamiquement alloués**, tandis que la **pile** est une structure de données en mémoire qui stocke **les données temporaires du programme**, telles que les variables locales et les appels de fonction.

En utilisant la pile, un programme peut stocker temporairement des variables et des adresses de retour lorsqu'il appelle une fonction. La pile est gérée automatiquement par le compilateur ou l'interpréteur du langage de programmation, qui ajoute et retire des éléments de la pile à mesure que les fonctions sont appelées et retournées.

Le tas, quant à lui, est utilisé pour allouer de la mémoire dynamiquement pendant l'exécution du programme. Le programmeur peut demander une quantité spécifique de mémoire à allouer sur le tas à l'aide de fonctions telles que malloc() et free(). Cela permet au programme de gérer la mémoire de manière plus flexible que la pile, mais cela nécessite également une gestion plus complexe de la mémoire pour éviter les fuites de mémoire et les problèmes de fragmentation.

En TypeScript, la gestion de la mémoire est gérée par le moteur JavaScript qui utilise un Garbage Collector pour libérer automatiquement la mémoire allouée pour les objets qui ne sont plus utilisés. Bien que les développeurs TypeScript n'aient pas besoin de gérer explicitement la mémoire comme dans les langages à faible niveau, tels que le C ou le C++, ils doivent être conscients de l'utilisation de la mémoire pour éviter les fuites de mémoire et les performances inutiles.

En PHP, la gestion de la mémoire est également gérée automatiquement par le moteur PHP. PHP utilise un système de gestion de mémoire par comptage de références pour allouer et libérer automatiquement la mémoire pour les variables et les objets. Cependant, les développeurs PHP peuvent également allouer de la mémoire dynamiquement à l'aide de fonctions telles que malloc() et free(), mais cela est rarement nécessaire.

En résumé, bien que la gestion de la mémoire puisse varier entre les langages de programmation, la compréhension des concepts de tas et de pile reste importante pour les développeurs de tous les langages. Les développeurs doivent être conscients de l'utilisation de la mémoire pour éviter les fuites de mémoire et les performances inutiles, mais cela peut être géré de manière différente selon les langages.