# Préparer le code pour la génération de setup

Pour tout les fichiers templates, nous utilisons une librairie qui s'appelle Jinja2 pour résoudre les balises dans les templates.

## Préparer l'application

### Définition de l'arboresence de dépendances

Dans le dossier _install à la racine de votre application, il faut créer/éditer le fichier `dependencies.json`.

Ce fichier servira de fichier de dépendance pour VAI.

Exemple de vMap :
```js
{
  // alias Apache
  "API_ALIAS": "/rest",
  "APP_ALIAS": "/vmap",
  "WMS_ALIAS": "/wms",
  // Environnement, laisser PROD dans 100% des cas
  "APP_ENV": "prod",
  // chemin pour Apache
  "HTTPSERVER_CONF_FILE": "vm_app_vmap.conf",
  "HTTPSERVER_PATH": "/etc/apache2",
  // hostname du serveur
  "HOSTNAME": "",
  // Chemin d'installation
  "INSTALL_PATH": "/var/www/vmap",
  // laisser full
  "INSTALL_TYPE": "full",
  // recalculer par VAI ça sert à rien
  "PLATFORM": "Linux",
  // paramètre de connexion à la base
  "POSTGRES_DB": "vmap",
  "POSTGRES_HOST": "dbhost",
  "POSTGRES_PASSWORD": "dbpasswd",
  "POSTGRES_PORT": 5432,
  "POSTGRES_USER": "dbuser",
  // données utilisable dans le sqlQueries
  "SQL_DATA": {
    "SRID": 2154,
    "DB_NAME": "vmap"
  },
  "application": {
    // OS autorisé pour VAI (attention cette liste est une sous liste contenue dans le script VAI ne pas mettre n'importe quoi)
    "operating_systems": [
      "Windows Server 2016",
      "Windows Server 2019",
      "Windows Server 2022",
      "Ubuntu 20",
      "Ubuntu 22",
      "Debian 11"
    ],
    // paramètre par défaut de l'application
    "default_description": "# Bienvenue dans vMap ! \n\nPlus d'information sur [http:\/\/www.veremes.com](http:\/\/www.veremes.com)",
    "default_theme": "6",
    // version de l'application 
    "branch": "2023.01.01",
    // nom de l'application
    "name": "vmap",
    // url git pour l'application (ne sert à rien)
    "url": "git@gitlab.veremes.net:Development/vitis-2020/applications/vmap.git",
    // utilisateur par défaut de l'application
    "users": {
      "admin": {
        "grouproles": [
          "vitis_user",
          "vitis_admin",
          "vmap_user",
          "vmap_admin"
        ],
        "groups": [
          "Cartes publiques"
        ],
        "login": true,
        "password": "admin",
        "roles": [
          "LOGIN",
          "CREATEROLE"
        ]
      }
    },
    // Vitis
    "framework": {
      "branch": "2023.02.01",
      "name": "vitis",
      "url": "git@gitlab.veremes.net:Development/vitis-2020/sources/vitis.git"
    },
    // modules de l'application
    "modules": [{
      "branch": "2023.01.01",
      "name": "module_vmap",
      "url": "git@gitlab.veremes.net:Development/vitis-2020/sources/module_vmap.git"
    }, {
      "branch": "2023.01.00",
      "name": "module_cadastre",
      "url": "git@gitlab.veremes.net:Development/vitis-2020/sources/module-cadastre-2020.git"
    }]
  }
}

```

```{Tip}
  Vous pouvez ajouter des clés à la racine si necessaire elle seront accessibles via le service settings dans les scripts python.
```

### Définition d'un README.md spécifique

Il suffit de mettre un fichier `INSTALL.md` dans le dossier _install pour qu'il soit copier en tant que README.md

### Fichier de properties du client

Dans `_install/templates` il faut définir le fichier de properties par défaut.

ex : 
```python
{
  {% if not client_properties_exists %}
  "status": "stable",
  "application": "{{application.name}}",
  "default_show_sidenav": true,
  "widget_public_connect_available": true,
  "map": {
    "default_feature_zoom": 15,
    "draw_style": {
      "draw_color": "rgba(54, 184, 255, 0.6)",
      "contour_color": "rgba(0, 0, 0, 0.4)",
      "contour_size": 2,
      "circle_radius": 7
    }
  },
  "documentation": [],
  {% endif %}
  "app_api_url": "https://[hostname]{{API_ALIAS}}/v2",
  "proxy_url": "https://[hostname]{{API_ALIAS}}/v2/proxy/proxy",
  "smallscreen_limit": 768,
  "version": "{{application.branch}}",
  "build": "{{APP_BUILD}}",
  "month_year": "{{MONTH_YEAR}}"
}
```

Le fichier évalué sera mergé avec le fichier existant pour récupérer des clés à créé dans ce fichier.

Ce qui se trouve dans la balise jinja ne sera évalué que pour les installations et pas pour les mises à jour.

### Template Apache principal

Par défaut c'est un fichier qui va inclure les autres templates par défaut.

ex : 
```python
{% include "vitis/vm_php_symfony.conf" with context %}

{% include "vitis/vm_vas.conf" with context %}

{% include "module_vmap/vm_wms.conf" with context %}

{% include "vitis/vm_client.conf" with context %}

```


### Scripts shell

Vous devez avoir un script mount_project.sh fonctionnel, car il sera utilisé par VAI pour créer l'arborescence finale de votre application lors de la génération du app.tgz.

Si le fichier buildengines.sh est présent il sera exécuté.

```{Warning}
  Je vous déconseille fortement de laisser le script buildengines.sh. La compilation avec Wine présente de gros désavantages et augmente le temps de génération des setups de plusieurs minutes.
```

## Préparer les modules

A la racine de chaque module se trouve aussi un dossier `_install`.
### Définition du fichier de dépendances

Le fichier dependencies.json à la racine du module permet de définir ce qui est necessaire au fonctionnement du module.

ex : 
```json
{
  "dependency_schema": {
    "s_vmap_2": {
      "path": "vas/src/Module/Vmap/sql/sqlQueries.xml"
    }
  },
  "users": {
    "mon_exemple": {
      "generate_password": true,
      "roles": [
        "LOGIN",
        "CREATEROLE",
        "INHERIT"
      ]
    }
  },
  "ressources": [
    {
      "name": "mapserver",
      "url": {
        "Linux": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/mapserver/7.6.4-gdal-3.4.3/linux/mapserver.tar.gz",
        "Windows": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/mapserver/7.6.4-gdal-3.4.3/windows/mapserver.zip"
      },
      "install_path": "vas/bin",
      "path": {
        "Linux": "vas/bin/mapserver/bin",
        "Windows": "vas/bin/mapserver"
      }
    }
  ],
  "jobs": [
    {
      "period": "0 0 * * *",
      "command_lin": "cd \"{{INSTALL_PATH}}/vas/engine/vitis/\"; ./exploitationService cleanFiles",
      "command_win": "cd \"{{INSTALL_PATH}}/vas/engine/vitis/\" & .\\exploitationService.exe cleanFiles"
    }
  ],
  "preinstall": false,
  "postinstall": true,
  "bak": [],
  "buildapp_cleaner": [
    "vas/src/Module/Vmap/postman",
    "client/dist/vitis/modules/vmap/components",
    "client/dist/vitis/modules/vmap/struct",
    "client/dist/vitis/modules/vmap/styles"
  ],
  "installer_cleaner": [
    "vas/bin/mapserver",

    "vas/src/Module/Vmap/postman",
    "vas/src/Module/Vmap/WebService",
    "vas/src/Module/Vmap/sql",
    "vas/src/Module/Vmap/services.yml",
    "vas/src/Module/Vmap/conf.sample"
  ]
}
```
Ce fichier de dépendance vous permet de définir les pré-requis et d'activer plusieurs options pour votre module.

1. dependency_schema : Permet de définir le nom du schéma postgres pour le controle de version et le chemin vers le fichier sqlQueries à exécuter.
1. users : Les utilisateurs necessaires au fonctionnement de l'application (utilisateur au sens vitis ou postgres)
1. ressources : Les binaires externes à l'application mais necessaires à son fonctionnement. ATTENTION : Si le lien n'est pas fonctionnel le setup ne pourra pas se générer, il est fortement conseiller de n'utiliser que des liens vers le bucket AWS S3 de développement.
1. jobs : Permet de définir des jobs cron pour le service vitis. Les requêtes ne fonctionnent pas à l'heure actuelle ce sera implémenté très bientôt.
1. preinstall & postinstall : Activation des script python de pre/post installation.
1. bak : Liste des fichiers à copier en .bak lors de mise à jour (par défaut les fichiers de conf sont déjà dedans)
1. buildapp_cleaner : Fichier/dossier à supprimer avant la génération du app.tgz (ex : fichier présent aprés la compilation mais inutile, ...)
1. installer_cleaner : Fichier/dossier à supprimer dans l'application existante avant d'installer la nouvelle version de l'application. L'écrasement n'effacera jamais des fichiers supprimés du dépot (ex : dépendance npm ou composer).

### Définition de templates spécifiques

Vous pouvez ajouter un dossier templates dans votre dossier _install.

Au moment de la génération du app.tgz, il sera copié à la racine du _install/templates de l'application, il sera renommé d'aprés le nom du module dans le dossier src de l'application.

ex : _install/templates/module_vmap/...

### Ajout d'un script de pré/post installation

Un script de pre/post installation doit respecter certaines conditions. 

Sa structure minimale doit être : 
```python
#!/usr/bin/env python3
# coding: utf-8

class Command:
  def __init__(self, **kwargs):
    self.settings = kwargs.get('settings')
    self.logging = kwargs.get('logging')
    self.env = kwargs.get('env')

  def execute(self):
    self.logging.info('Mon module pre/postinstall')
```

Le python utilisé pour exécuter le script est compilé dans l'exécutable de l'installer, vous ne pourrez donc pas utiliser n'importe quelle librairie.

```{Tip}
  Pour ajouter une libraire, il faut générer un EC2 avec l'AMI de VAI, se connecter en ssh dessus avec la clé SSH Jenkins, installer la librairie que l'on souhaite utiliser dans l'OS principal et dans Wine (c'est là que ça devient problèmatique). Puis il faut générer une nouvelle AMI et modifier l'AMI dans le dépôt Git des scripts Jenkins. Opération à réaliser sur une branche spécifique du dépôt pour tester avant de merger dans master.
```

Actuellement vous n'avez que quelques services disponibles via les kwargs : 
- settings : qui permet de récupérer le contenu du dependencies.json (on parle de la version finale qui est dans le setup)
- logging : qui permet d'écrire dans les logs de vai
- env : qui permet d'interagir avec les templates
- db : qui permet de lancer des requêtes en base


En dehors de cette stucture et de ces limitations vous pouvez faire ce que vous voulez dans le fichier.

```{Note}
  Il faut garder à l'esprit que ces scripts devront être rejouables, et savoir par eux-même s'ils devront intervenir en installation ou mise à jour.
  Il faut prévoir de tester l'existence de fichiers quand c'est necessaire, vérifier des informations en base, ...
```

Pour des cas d'utilisations, je vous conseille de regarder le module vMap ou GTF.

## Préparer un module externe

En ce qui concerne les modules externes le fichier de dépendances se présente comme le fichier d'un module classique fusionné avec celui de l'application.

Il se nommera dependencies_mod.json, il comportera :
- les infos pour accéder à l'application, interagir avec la base, ...
- Le chemin vers le sqlQueries devra être renseigné via la clé archive_path en supplément de path qui aura un autre usage dans ce cas (point qui pourra évoluer)
- Des pré-requis sur des versions de modules pourront être spécifiés

ex : 
```json
{
  "APP_ENV": "prod",
  "INSTALL_PATH": "/var/www/vmap",
  "HTTPSERVER_PATH": "/etc/apache2",
  "INSTALL_TYPE": "full",
  "PLATFORM": "Linux",
  "POSTGRES_DB": "vmap",
  "POSTGRES_HOST": "dbhost",
  "POSTGRES_PASSWORD": "dbpasswd",
  "POSTGRES_PORT": 5432,
  "POSTGRES_USER": "dbuser",
  "module": {
    "name" : "rismajic",
    "branch" : "master",
    "ws_name" : "RisMajic",
    "requirements" : {
      "s_vitis" : "2022.08.05",
      "s_vmap_2" : "2022.05.00"
    },
    "dependency_schema": {
      "s_rismajic": {
        "path": "vas/src/Module/RisMajic/sql/sqlQueries.xml",
        "archive_path": "web_services/sql/sqlQueries.xml"
      }
    },
    "users": {},
    "jobs": [],
    "ressources": [],
    "preinstall": false,
    "postinstall": false,
    "bak": [],
    "buildapp_cleaner": [
      "web_services/postman",
      "spec"
    ],
    "installer_cleaner": [
      "vas/src/Module/RisMajic/postman",
      "vas/src/Module/RisMajic/WebService",
      "vas/src/Module/RisMajic/sql",
      "vas/src/Module/RisMajic/services.yml",
      "vas/src/Module/RisMajic/conf.sample"
    ]
  }
}
```