# PgAdmin 4

PgAdmin 4 est un outil open source développé comme une application Web Jquery/Bootstrap ainsi que d'un serveur développé en Python avec la librairie Flask.

Le but de cet outil est de simplifier l'administration et la consultation de serveur de BDD Postgresql pour les responsables d'infrastructures et les développeurs.

## Consultation et modification de la donnée

![interface consultation](./images/pgadmin_consultation.png)

Vous pouvez voir la donnée d'une table/vue soit par le bouton au dessus de l'arbre, soit via le menu contextuel en faisant un clique droit sur l'objet en question.

## Visualisation de données spatiale

Sur une colonne de type géométrie, il y aura un bouton permettant la visualisation des géométrie WKB sur une carte OpenLayers comportant un fond OSM.

Le fond de plan ne sera présent que si la géométrie est sotcké en WGS84 (4326).

## Administration de la base de données

### Visualiser la configuration de la base

![interface configuration](./images/interface_configuration.png)

En sélectionnant le serveur, il est possible d'accéder à un tableau permettant de voir la configuration postgresQl du serveur.

Ce qui peut être pratique pour voir par exemple l'algorithme de hashage des mots de passe par exemple.

### Créer une sauvegarde

Il existe deux binaires permettant de réaliser ces opérations, pg_dump et pg_dumpall. L'utilisation de ces deux commande peut trés vite être casse tête à cause du nombre de paramètre à passer dans ces commandes. 

Heureusement pgAdmin permet de réaliser ces opérations.

En faisant un clique droit sur un serveur, une base, un schéma ou une table on peut lancer la commande pg_dump (pg_dumpall dans le cas du serveur), ce qui va permettre d'ouvrir une modale de configuration qui va vous permettre de lancer la création de la sauvegarde.

Il est possible de sauvegarder les roles de connexion, la structure de données et les données (tout ces objets ou seulement certains, les roles ne sont récupérable qu'en sauvegardant le serveur).

Il est possible de générer différents types de fichier, comme le format SQL qui va permettre d'avoir des requêtes SQL textuelle, ou un fichier backup binaire qui sera plus simple à restaurer.

La commande sera ensuite exécuté de manière asynchrone, ce qui vous permettra d'utiliser l'application pendant ce temps. La commande sera écrit dans el log ce qui vous permettra de la copier pour sauvegarder régulièrement votre base via une tâche CRON par exemple.

### Restaurer une sauvegarde

Comme pour la sauvegarde il existe un binaire pg_restore qui permet de réaliser cette opération.

Comme pour la sauvegarde en réalisant un clique droit sur l'objet à restaurer on peut configurer la commande et la lancer de manière asynchrone.

Le format binaire est préfèrable pour la restauration car plus rapide et plus léger.

## Génération de code SQL

Une section permet de récupérer le code SQL de l'objet sélectionné.

![interface de base](./images/interface_pgadmin.png)

Via le menu contextuel sur une table ou une vue il est aussi possible de récupérer une requêtes simple sur cet objet (sous-menu `Scripts`).

## Exécution de code SQL

En cliquant sur un des boutons en haut de l'arbre de gauche, on peut ouvrir un fenère d'éxécution SQL sur la base sélectionnée (ou la base contenant l'objet sélectionné s'il s'agit d'un sous objet de la base).

Cet outil se comporte comme une éxécution du binaire psql, certaines commande ont un impact sur l'outil de manière définitive, il faut alors ouvrir un nouvel outil pour revenir à l'état initial (comme le set Role par exemple).

### Conseil d'utilisation sur une base de production

Il pourra vous arriver de lancer des requêtes UPDATE ou DELETE sur la base lors de mise à jour ou d'opération de maintenance. Il est préférable dans ce cas de désactivé l'auto-commit.

Dans ce cas, vous pouvez lancer votre commande, elle n'aura aucun impact sur la base tant que la transction n'est pas explicitement résolue.
Le commit ou le rollback se font d'un simple bouton sur l'interface.

Lorsque vous fermer l'outil de requête, il va vous demander si vous voulez sauvegarder le code sql saisie (comme d'habitude), puis si la transaction n'est aps résolue, alors il ouvrira une deuxième fenètre de confirmation pour savoir si vous voulez commit ou rollback la transaction.

## Utilisation du Explain/Analyze

PgAdmin vous permet d'obtenir un diagramme bien plus simple à exploiter que le retour classique d'un Explain Analyze.

exmple la requête ci-dessous : 

```sql
SELECT "user".user_id,
    "user".login,
    "user".name,
    "user".email,
    "user".company,
    "user".department,
    "user".ip_constraint,
    domain.domain,
    "user".domain_id,
    "user".last_connection,
    "user".restriction,
        CASE
            WHEN array_to_string(ARRAY( SELECT pg_group.groname
               FROM pg_group
                 JOIN pg_roles ON pg_roles.oid = ANY (pg_group.grolist)
              WHERE "user".login::name = pg_roles.rolname AND (pg_group.groname = 'vitis_admin'::name OR pg_group.groname = 'vitis_user'::name)), '|'::text) = 'vitis_user|vitis_admin'::text OR array_to_string(ARRAY( SELECT pg_group.groname
               FROM pg_group
                 JOIN pg_roles ON pg_roles.oid = ANY (pg_group.grolist)
              WHERE "user".login::name = pg_roles.rolname AND (pg_group.groname = 'vitis_admin'::name OR pg_group.groname = 'vitis_user'::name)), '|'::text) = 'vitis_admin|vitis_user'::text THEN 'admin'::text
            WHEN array_to_string(ARRAY( SELECT pg_group.groname
               FROM pg_group
                 JOIN pg_roles ON pg_roles.oid = ANY (pg_group.grolist)
              WHERE "user".login::name = pg_roles.rolname AND (pg_group.groname = 'vitis_admin'::name OR pg_group.groname = 'vitis_user'::name)), '|'::text) = 'vitis_user'::text THEN 'user'::text
            ELSE ''::text
        END AS role,
    "user".dataencrypt,
    "user".secretkey,
    "user".billinggroup_id,
    billinggroup.billinggroup,
    "user".phone,
    "user".acceptnotification,
    "user".timezone_id,
    "user".formatdate_id,
    rt_formatdate.formatdate,
    "user".comp_data
   FROM s_vitis."user"
     LEFT JOIN s_vitis.domain ON "user".domain_id = domain.domain_id
     LEFT JOIN s_vitis.billinggroup ON "user".billinggroup_id = billinggroup.billinggroup_id
     LEFT JOIN s_vitis.rt_formatdate ON "user".formatdate_id::text = rt_formatdate.formatdate_id::text
  WHERE "user".login::name = "current_user"() OR s_vitis.is_vitis_admin();
```

va donner le diagramme suivant : 

![Explain](./images/explain_example.svg)