# Présentation

VAI s'appuie sur [PyInstaller](https://pyinstaller.org/en/stable/), qui permet de générer un executable à partir d'un code python et de l'interpréteur python présent sur la machine de compilation.

VAI est un script python permettant d'installer une application Web Vitis, c'est à dire qu'il va réaliser les taches suivantes : 
- Vérification des pré-requis sur le serveur :
  - Espace disque suffisant
  - OS compatible
  - Présence de Postgres, Postgis, Apache
- Exécution des scripts de pré-installation
- Sauvegarde des fichiers sensibles
- Activation d'extension postgres (postgis, pgcrypto)
- Mise à jour de la base de données
- Création d'utilisateurs dans la base et/ou l'application
- Mise à jour du code source de l'application
- Suppression de fichiers à ne pas publier en production
- Configuration de l'application
- Configuration d'Apache
- Génération des clés de chiffrement nécessaire au fonctionnement de l'application.
- Installation de ressources et d'exécutables externe
- Exécution de script de post-installation

Le livrable de VAI que l'on appelle aussi setup ou installer, se compose de plusieurs fichiers : 
- installer/installer.exe : binaire qui contient l'interpréteur Python et le script d'installation
- app.tgz/mod.tgz : code source et ressources complémentaires à installer
- dependencies.json : fichier de configuration de l'installation
- Readme.md : fichier expliquant ce qu'il faut parametrer sur le dependencies.json

Pour installer une application il suffit d'éditer le dependencies.json en fonction de ce qui est demandé pour l'application à installer, et de lancer le binaire `installer(.exe)` (en sudo pour linux ou administrateur pour windows). 
Suite à l'exécution, un fichier de log sera généré contenant ce qui est remonté dans la console pendant l'exécution.


Exemple de dependencies.json : 

```json
{
  "API_ALIAS": "/rest",
  "APP_ALIAS": "/gtf",
  "APP_BUILD": 8188,
  "APP_ENV": "prod",
  "HOSTNAME": "",
  "HTTPSERVER_CONF_FILE": "vm_app_gtf.conf",
  "HTTPSERVER_PATH": "/etc/apache2",
  "INSTALL_PATH": "/var/www/gtf",
  "INSTALL_TYPE": "full",
  "POSTGRES_DB": "gtf",
  "POSTGRES_HOST": "dbhost",
  "POSTGRES_PASSWORD": "dbpasswd",
  "POSTGRES_PORT": 5432,
  "POSTGRES_USER": "dbuser",
  "application": {
    "branch": "2022.02.06",
    "default_description": "# Bienvenue dans GTF !\n\nCette application vous permet d'ex\u00e9cuter des traitements publi\u00e9s par l'administrateur du service et de recevoir le r\u00e9sultat sous forme de fichiers dans votre bo\u00eete mail.\n\nLes traitements propos\u00e9s peuvent \u00eatre de toute nature : contr\u00f4le qualit\u00e9, extraction, chargement, g\u00e9n\u00e9ration de rapport, conversion...\n\nSi vous n'\u00eates pas encore utilisateur, vous pouvez cr\u00e9er un compte pour acc\u00e9der au service avec des droits limit\u00e9s en cliquant sur le lien \"Cr\u00e9er un compte.\"\n\nSi vous \u00eates administrateur, vous pouvez d\u00e9finir le texte d'accueil qui va appara\u00eetre ici dans le menu Configuration, ou laissez cet emplacement vide. \n\nPlus d'information sur [http://www.veremes.com](http://www.veremes.com)",
    "default_theme": "1",
    "framework": {
      "bak": [
        "vas/engine/vitis/conf/jobs.json"
      ],
      "branch": "2022.08.04",
      "dependency_schema": {
        "s_vitis": {
          "path": "vas/src/Module/Vitis/sql/sqlQueries.xml"
        }
      },
      "installer_cleaner": [
        "vas/bin/php",
        "vas/bin/phantomjs",
        "vas/src/Module/Vitis/postman",
        "vas/src/Module/Vitis/WebService",
        "vas/src/Module/Vitis/sql",
        "vas/src/Module/Vitis/services.yml",
        "vas/src/Module/Vitis/conf.sample"
      ],
      "jobs": [
        {
          "command_lin": "cd \"{{INSTALL_PATH}}/vas/engine/vitis/\"; ./exploitationService cleanFiles",
          "command_win": "cd \"{{INSTALL_PATH}}/vas/engine/vitis/\" & .\\exploitationService.exe cleanFiles",
          "period": "0 0 * * *"
        }
      ],
      "name": "vitis",
      "postinstall": true,
      "preinstall": false,
      "ressources": [
        {
          "install_path": "vas/bin",
          "name": "nssm",
          "path": {
            "Windows": "vas/bin/nssm-2.24-101-g897c7ad/win{{SYS_TYPE}}/nssm.exe"
          },
          "url": {
            "Windows": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/nssm/windows/nssm-2.24-101-g897c7ad.zip"
          }
        },
        {
          "install_path": "vas/bin",
          "name": "phantomjs",
          "path": {
            "Linux": "vas/bin/phantomjs/bin/phantomjs",
            "Windows": "vas/bin/phantomjs/bin/phantomjs.exe"
          },
          "url": {
            "Linux": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/phantomjs-2.1.1/linux/phantomjs.tar.gz",
            "Windows": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/phantomjs-2.1.1/windows/phantomjs.zip"
          }
        },
        {
          "install_path": "vas/engine/vitis",
          "name": "vitis_engine",
          "url": {
            "Linux": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/engines/vitis/linux/2022_08_02.tar.gz",
            "Windows": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/engines/vitis/windows/2022_08_02.zip"
          }
        },
        {
          "install_path": "vas/bin",
          "name": "php",
          "path": {
            "Linux": "vas/bin/php/bin/php-cgi",
            "Windows": "vas/bin/php/php-cgi.exe"
          },
          "url": {
            "Linux": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/php/8.1/8.1.8/linux/php.tar.gz",
            "Windows": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/php/8.1/8.1.8/windows/php.zip"
          }
        }
      ],
      "users": {
        "u_vitis": {
          "generate_password": true,
          "roles": [
            "LOGIN",
            "CREATEROLE",
            "INHERIT"
          ]
        }
      }
    },
    "modules": [
      {
        "bak": [
          "vas/engine/gtf/conf/properties.json"
        ],
        "branch": "2022.02.03",
        "dependency_schema": {
          "s_gtf": {
            "path": "vas/src/Module/Gtf/sql/sqlQueries.xml"
          }
        },
        "installer_cleaner": [
          "vas/src/Module/Gtf/postman",
          "vas/src/Module/Gtf/WebService",
          "vas/src/Module/Gtf/sql",
          "vas/src/Module/Gtf/services.yml",
          "vas/src/Module/Gtf/conf.sample"
        ],
        "jobs": [
          {
            "command_lin": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\"; ./engine startOrder --nc",
            "command_win": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\" & .\\engine.exe startOrder --nc",
            "period": "*/5 * * * * *",
            "user_lin": "www-data"
          },
          {
            "command_lin": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\"; ./engine checkOrder --nc --nt",
            "command_win": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\" & .\\engine.exe checkOrder --nc --nt",
            "period": "*/10 * * * * *",
            "user_lin": "www-data"
          },
          {
            "command_lin": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\"; ./engine startInstance --nc",
            "command_win": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\" & .\\engine.exe startInstance --nc",
            "period": "0 * * * * *",
            "user_lin": "www-data"
          },
          {
            "command_lin": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\"; ./engine stopInstance --nc",
            "command_win": "cd \"{{INSTALL_PATH}}/vas/engine/gtf/\" & .\\engine.exe stopInstance --nc",
            "period": "0 5 * * * *",
            "user_lin": "www-data"
          }
        ],
        "name": "module_gtf",
        "postinstall": true,
        "preinstall": true,
        "ressources": [
          {
            "install_path": "vas/engine/gtf",
            "name": "gtf_engine",
            "url": {
              "Linux": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/engines/module_gtf/linux/2022_02_03.tar.gz",
              "Windows": "https://veremes-dev-ressources.s3.eu-west-3.amazonaws.com/dependencies/engines/module_gtf/windows/2022_02_03.zip"
            }
          }
        ],
        "users": {
          "u_scheduler": {
            "generate_password": true,
            "roles": [
              "LOGIN",
              "INHERIT"
            ]
          }
        }
      }
    ],
    "name": "gtf",
    "operating_systems": [
      "Windows Server 2016",
      "Windows Server 2019",
      "Windows Server 2022",
      "Ubuntu 20",
      "Debian 11"
    ],
    "users": {
      "admin": {
        "grouproles": [
          "vitis_user",
          "vitis_admin",
          "gtf_user",
          "gtf_admin"
        ],
        "login": true,
        "password": "admin",
        "roles": [
          "LOGIN",
          "CREATEROLE"
        ]
      }
    }
  }
}
```